require 'thor'
require 'awesome_print'
require 'colorize'
require 'aws-sdk-core'
require 'yaml'
require 'customresource/elastictranscoder/version'
require 'aws/ec2/instance_data'

module CustomResource
  module ElasticTranscoder

    class Cli < Thor
      class_option :verbose,      :type => :boolean
      class_option :debug,        :type => :boolean
      class_option :trace,        :type => :boolean
      class_option :log_level,    :type => :string, :banner => 'Log level ([:trace, :debug, :info, :step, :warn, :error, :fatal, :todo])'
      class_option :log_file,     :type => :string
      class_option :inifile,      :type => :string
      class_option :input,        :type => :string
      class_option :ip_address,   :type => :string

      no_commands do

        require 'dldinternet/mixlib/thor/no_commands'
        include DLDInternet::MixLib::Thor::No_Commands

        require 'customresource/elastictranscoder/mixins/cli'
        include CustomResource::ElasticTranscoder::MixIns::Cli

        require 'customresource/elastictranscoder/mixins/actions'
        include CustomResource::ElasticTranscoder::MixIns::Actions

      end # no_commands

      def initialize(args = [], local_options = {}, config = {})
        super(args,local_options,config)
        @log_level = :step
      end

      desc 'version', 'display current version'
      def version()
        puts ::CustomResource::ElasticTranscoder::VERSION
        exit 0
      end

      desc 'preset', 'CRUD for presets'
      def preset()
        process('Presets')
      end

      desc 'pipeline', 'CRUD for pipelines'
      def pipeline()
        process('Pipelines')
      end

      desc 'job', 'CRUD for jobs'
      def job()
        process('Jobs')
      end

    end
  end
end

